<?php
/**
 * Plugin Name: Easy Table of Contents
 * Plugin URI: http://w-shadow.com/blog/2012/06/17/table-of-contents-plugin-tutorial/
 * Version: 1.0
 * Author: Janis Elsts
 * Author URI: http://w-shadow.com/
 * Description: Insert a table of contents into your posts by using the [toc] shortcode.
 */

class EasyTOC {
	const TOC_POSITION_MARKER = '/--Put_TOC_here--/';

	private $tocSettings;
	private $tocEntries;

	public function __construct() {
		add_shortcode('toc', array($this, 'doTocShortcode'));
		add_action('the_content', array($this, 'insertToc'), 20); //Run after shortcodes.
		add_action('wp_enqueue_scripts', array($this, 'registerStyle'));
	}

	public function doTocShortcode($attributes) {
		//Only display the TOC on single posts and pages, not archives.
		if ( !(is_single() || is_page()) ) {
			return '';
		}

		$attributes = shortcode_atts(
			array(
				'title' => '',
				'class' => '',
				'headings' => '1,2,3,4,5,6',
			),
			$attributes
		);
		$attributes['headings'] = array_map('intval', explode(',', $attributes['headings']));

		//We'll use these settings later, in insertToc().
		$this->tocSettings = $attributes;

		//Add our stylesheet to pages that contain a TOC.
		wp_enqueue_style('easy-toc-style');

		return self::TOC_POSITION_MARKER;
	}

	public function insertToc($content) {
		//Replace the marker with the actual table of contents.
		if (strpos($content, self::TOC_POSITION_MARKER) !== false) {
			list($entries, $content) = $this->collectTocEntries($content, $this->tocSettings['headings']);
			$toc = $this->buildToc($entries, $this->tocSettings['title'], $this->tocSettings['class']);
			$content = str_replace(self::TOC_POSITION_MARKER, $toc, $content);
		}
		return $content;
	}

	private function collectTocEntries($content, $headingLevels) {
		$this->tocEntries = array();

		$pattern = sprintf(
			'@<h(?P<level>%s)[^>]*>(?P<content>.*?)</h(?P=level)>@si',
			implode('|', $headingLevels)
		);
		$content = preg_replace_callback($pattern, array($this, 'processHeading'), $content);

		return array($this->tocEntries, $content);
	}

	public function processHeading($matches) {
		$anchor = $this->addTocEntry($matches['content'], $matches['level']);
		return $this->addAnchorTarget($matches[0], $anchor);
	}

	private function addTocEntry($title, $level) {
		$title = strip_tags($title);
		$level = intval($level);

		//Each TOC entry needs a unique anchor.
		$anchor = sanitize_title($title);
		if ( array_key_exists($anchor, $this->tocEntries) ) {
			$anchor = $anchor . '-' . count($this->tocEntries);
		}

		$this->tocEntries[$anchor] = array($level, $title);
		return $anchor;
	}

	private function addAnchorTarget($heading, $anchor) {
		//For in-page links, <elem id="..."> is preferred to <a name="...">.
		//See http://stackoverflow.com/questions/484719/html-anchors-with-name-or-id
		$target = sprintf('<span id="%s"></span>', esc_attr($anchor));
		return $target . $heading;
	}

	private function buildToc($entries, $title = '', $class = '') {
		//No entries - no TOC.
		if ( empty($entries) ) {
			return '';
		}

		$class = 'toc ' . $class;
		$html = '<div class="' . esc_attr($class) . '">';

		if ( !empty($title) ) {
			$html .= '<span class="toc-title">' . $title . '</span>';
		}

		//Determine the minimum heading level.
		$minLevel = PHP_INT_MAX;
		foreach($entries as $entry) {
			$minLevel = min($entry[0], $minLevel);
		}

		$currentLevel = $minLevel - 1;

		foreach($entries as $anchor => $entry) {
			list($level, $title) = $entry;

			if ( $currentLevel < $level ) {
				$html .= str_repeat('<ul><li>', $level - $currentLevel);
			} else {
				$html .= str_repeat('</li></ul>', $currentLevel - $level) . '<li>';
			}
			$currentLevel = $level;

			$html .= sprintf('<a href="#%s">%s</a>', esc_attr($anchor), $title);
		}

		//Close any open lists.
		if ( $currentLevel > $minLevel - 1 ) {
			$html .= str_repeat('</li></ul>', $currentLevel - $minLevel + 1);
		}

		$html .= '</div>';
		return $html;
	}

	public function registerStyle() {
		wp_register_style('easy-toc-style', plugins_url('style.css', __FILE__), array(), '20120617');
	}
}

new EasyTOC();